from dataclasses import dataclass
from datetime import datetime, timedelta
from .const import DEFAULT_OVERRIDE, MONTHS

@dataclass(frozen=False)
class Timer:
    expire:datetime = datetime.now()

    @property
    def is_override(self) -> bool:
        return self.expire > datetime.now()

    @property
    def override_string(self) -> str:
        if self.expire.day != datetime.now().day:
            _dt = f"{self.expire.day} {MONTHS[self.expire.month]} - {self.expire.hour}:{self.expire.minute}"
        else:
            _dt = f"{self.expire.hour}:{self.expire.minute}"
        return f"Nonhours ignored until {_dt}"

    def update(self, value_in_hours:int=DEFAULT_OVERRIDE):
        if not self.is_override:
            self.expire = datetime.now()
        try:
            assert isinstance(value_in_hours, int)
        except AssertionError as a:
            return
        value_in_seconds = value_in_hours * 3600
        self.expire += timedelta(seconds=value_in_seconds)



from ast import Pow
import time
from .power_reading import PowerReading


class SessionPrice:
    def __init__(self) -> None:
        self._total_price: int = 0
        self._price: int = 0
        self._current_power: int = 0
        self._total_energy: float = 0
        self._current_time: int = 0
        self._time_delta: int = 0
        self._readings: list = []

    @property
    def total_energy(self) -> float:
        return round(self.get_status()["energy"]["value"], 3)

    def reset(self):
        self.__init__()

    def terminate(self, mock_time: float=None):
        print("called terminate")
        self.update_power_reading(0, mock_time)
        self.get_status()

    def set_status(self) -> None:
        for i in self._readings:
            self._total_energy += i.reading_integral
            self._total_price += i.reading_cost

    def get_status(self) -> dict:
        self._total_energy = 0
        self._total_price = 0
        self.set_status()
        return {
            "energy": {
                "value": self._total_energy,
                "unit": "kWh"
            },
            "price": self._total_price
        }

    def update_power_reading(self, power: any, mock_time: float=None):
        self._set_delta(mock_time)
        p = PowerReading(self._price, self._current_power, self._time_delta)
        self._readings.append(p)
        try:
            self._current_power = float(power)
        except:
            self._current_power = 0

    def update_price(self, price: any, mock_time: float=None):
        if self._current_power > 0:
            self.update_power_reading(
                power=self._current_power,
                mock_time=mock_time
            )
        try:
            self._price = float(price)
        except:
            self._price = 0

    def _set_delta(self, mock_time: float=None) -> None:
        now = mock_time or time.time()
        self._time_delta = (now - self._current_time)
        self._current_time = now

    @property
    def readings(self) -> list:
        return self._readings

    @property
    def total_price(self) -> float:
        return round(self.get_status()["price"], 3)

    @total_price.setter
    def total_price(self, val):
        self.total_price = val



from .hoursbase import Hours

class PriceAwareHours(Hours):
    def __init__(
            self,
            hass,
            hub,
            absolute_top_price: float = None,
            min_price: float = 0,
            cautionhour_type: str = CAUTIONHOURTYPE_INTERMEDIATE,
            allow_top_up: bool = False
    ):
        self._hub = hub
        self._cautionhour_type = CAUTIONHOURTYPE_DICT[cautionhour_type]
        self._cautionhour_type_string = cautionhour_type
        self._core = core_hours(
            self._set_absolute_top_price(absolute_top_price),
            min_price,
            self._cautionhour_type, allow_top_up
        )
        self._hass = hass
        self._prices = []
        self._nordpool_entity = None
        self._nordpool_currency = ""
        self._is_initialized = False
        self._setup_nordpool()
        super().__init__(True)

    @property
    def options(self):
        return self._core.options

    @property
    def dynamic_caution_hours(self) -> dict:
        return self._core.dynamic_caution_hours

    @property
    def cautionhour_type_string(self) -> str:
        return self._cautionhour_type_string

    @property
    def non_hours(self) -> list:
        return self._core.non_hours

    @property
    def caution_hours(self) -> list:
        return self._core.caution_hours

    @property
    def absolute_top_price(self):
        return self._core.absolute_top_price

    @property
    def min_price(self):
        return self._core.min_price

    @property
    def nordpool_entity(self) -> str:
        return self._nordpool_entity

    @nordpool_entity.setter
    def nordpool_entity(self, val):
        self._nordpool_entity = val

    @property
    def prices(self) -> list:
        return self._core.prices

    @prices.setter
    def prices(self, val):
        self._core.prices = val

    @property
    def prices_tomorrow(self) -> list:
        return self._core.prices_tomorrow

    @prices_tomorrow.setter
    def prices_tomorrow(self, val):
        self._core.prices_tomorrow = val

    @property
    def currency(self):
        return self._nordpool_currency

    @currency.setter
    def currency(self, val):
        self._nordpool_currency = val

    @property
    def is_initialized(self) -> bool:
        if self.prices is not None:
            if len(self.prices) > 0:
                if self._is_initialized is False:
                    self._is_initialized = True
                    _LOGGER.debug("Hourselection has initialized")
                return True
        return False

    def get_average_kwh_price(self):
        if self._is_initialized:
            try:
                return self._core.get_average_kwh_price()
            except ZeroDivisionError as e:
                _LOGGER.warning(e)
        return 0

    def get_total_charge(self):
        if self._is_initialized:
            try:
                return self._core.get_total_charge(self._hub.currentpeak.value)
            except ZeroDivisionError as e:
                _LOGGER.warning(e)
        return 0

    def update_nordpool(self):
        ret = self._hass.states.get(self.nordpool_entity)
        if ret is not None:
            ret_attr = list(ret.attributes.get("today"))
            ret_attr_tomorrow = list(ret.attributes.get("tomorrow"))
            ret_attr_currency = str(ret.attributes.get("currency"))
            self.currency = ret_attr_currency
            self.prices = ret_attr
            self.prices_tomorrow = ret_attr_tomorrow
        else:
            _LOGGER.error("could not get nordpool-prices")

    def _setup_nordpool(self):
        try:
            entities = template.integration_entities(self._hass, NORDPOOL)
            if len(entities) < 1:
                raise Exception("no entities found for Nordpool.")
            if len(entities) == 1:
                self.nordpool_entity = entities[0]
                self.update_nordpool()
            else:
                raise Exception("more than one Nordpool entity found. Cannot continue.")
        except Exception as e:
            msg = f"Peaqev was unable to get a Nordpool-entity. Disabling Priceawareness: {e}"
            _LOGGER.error(msg)

    @staticmethod
    def _set_absolute_top_price(_input) -> float:
        if _input is None or _input <= 0:
            return float("inf")
        return _input


from datetime import datetime

def _convert_quarterly_minutes(minute:int, is_quarterly:bool) -> int:
    if is_quarterly is True:
        return (minute%15) * 4
    else:
        return minute

def parse_datetime(dtstring:str) -> datetime:
    pass

def nametoid(input_string) -> str:
    return input_string.lower().replace(" ", "_").replace(",", "")

def try_parse(input_string:str, parsetype:type):
    try:
        ret = parsetype(input_string)
        return ret
    except Exception as e:
        return False


from dataclasses import dataclass, field
from datetime import datetime
from abc import abstractmethod
from .const import NON_HOUR, CAUTION_HOUR, CHARGING_PERMITTED

# class Hours:
#     def __init__(
#             self,
#             price_aware: bool,
#             non_hours: list = None,
#             caution_hours: list = None
#     ):
#         self._non_hours = non_hours
#         self._caution_hours = caution_hours
#         self._price_aware = price_aware

#     @property
#     def state(self) -> str:
#         if datetime.now().hour in self.non_hours:
#             return NON_HOUR
#         if datetime.now().hour in self.caution_hours:
#             return CAUTION_HOUR
#         return CHARGING_PERMITTED

#     @property
#     def price_aware(self) -> bool:
#         return self._price_aware

#     @property
#     def non_hours(self):
#         return self._non_hours

#     @non_hours.setter
#     def non_hours(self, val):
#         self._non_hours = val

#     @property
#     def caution_hours(self):
#         return self._caution_hours

#     @caution_hours.setter
#     def caution_hours(self, val):
#         self._caution_hours = val

#     @property
#     @abstractmethod
#     def nordpool_entity(self):
#         pass

#     @abstractmethod
#     def update_nordpool(self) -> None:
#         pass

#     @property
#     @abstractmethod
#     def dynamic_caution_hours(self) -> dict:
#         pass

#     @property
#     @abstractmethod
#     def options(self):
#         pass

@dataclass
class Hours:
    price_aware: bool
    non_hours: list = None
    caution_hours: list = None

    @property
    def state(self) -> str:
        if datetime.now().hour in self.non_hours:
            return NON_HOUR
        if datetime.now().hour in self.caution_hours:
            return CAUTION_HOUR
        return CHARGING_PERMITTED

    @property
    @abstractmethod
    def nordpool_entity(self):
        pass

    @abstractmethod
    def update_nordpool(self) -> None:
        pass

    @property
    @abstractmethod
    def dynamic_caution_hours(self) -> dict:
        pass

    @property
    @abstractmethod
    def options(self):
        pass